@@ -5,12 +5,12 @@ SUFFIX = ".md2"
 
 
 @gethashcli(NAME, SUFFIX)
-def main(check, files, **kwargs):
+def main(files, **kwargs):
     """Generate and check MD2."""
 
     from Cryptodome.Hash.MD2 import new as H
 
-    script_main(H(), check, files, **kwargs)
+    script_main(H(), files, **kwargs)
 
 
 if __name__ == "__main__":
