@@ -5,12 +5,12 @@ SUFFIX = ".md5"
 
 
 @gethashcli(NAME, SUFFIX)
-def main(check, files, **kwargs):
+def main(files, **kwargs):
     """Generate and check MD5."""
 
     from hashlib import md5 as H
 
-    script_main(H(), check, files, **kwargs)
+    script_main(H(), files, **kwargs)
 
 
 if __name__ == "__main__":
