@@ -5,12 +5,12 @@ SUFFIX = ".ripemd160"
 
 
 @gethashcli(NAME, SUFFIX)
-def main(check, files, **kwargs):
+def main(files, **kwargs):
     """Generate and check RIPEMD160."""
 
     from Cryptodome.Hash.RIPEMD160 import new as H
 
-    script_main(H(), check, files, **kwargs)
+    script_main(H(), files, **kwargs)
 
 
 if __name__ == "__main__":
