@@ -5,12 +5,12 @@ SUFFIX = ".sha256"
 
 
 @gethashcli(NAME, SUFFIX)
-def main(check, files, **kwargs):
+def main(files, **kwargs):
     """Generate and check SHA256."""
 
     from hashlib import sha256 as H
 
-    script_main(H(), check, files, **kwargs)
+    script_main(H(), files, **kwargs)
 
 
 if __name__ == "__main__":
