@@ -12,6 +12,8 @@ __all__ = [
     "ParseHashLineError",
     "CheckHashLineError",
     "Hasher",
+    "HashFileReader",
+    "HashFileWriter",
     "format_hash_line",
     "parse_hash_line",
     "generate_hash_line",
@@ -44,8 +46,8 @@ class CheckHashLineError(ValueError):
 class Hasher(object):
     """General hash value generator.
 
-    Generate hash values using given hash context prototype. A progressbar
-    based on ``tqdm`` is also available.
+    Generate hash values using given hash context prototype. Note that a
+    ``tqdm`` progressbar is also available.
 
     Parameters
     ----------
@@ -170,30 +172,169 @@ class Hasher(object):
     __call__ = hash
 
 
-def format_hash_line(hash_value, path):
+class HashFileReader(object):
+    """General hash file reader.
+
+    Parameters
+    ----------
+    filepath : str or path-like
+        The path of a hash file.
+    """
+
+    def __init__(self, filepath):
+        self.name = filepath
+        self.file = open(filepath, "r", encoding="utf-8")
+
+    def close(self):
+        self.file.close()
+
+    def read_hash_line(self):
+        """Read hash line.
+
+        Returns
+        -------
+        hash_line : str
+            A line of *hash* and *filename* with GNU Coreutils style.
+        """
+
+        while True:
+            line = self.file.readline()
+            if line.startswith("#"):
+                continue
+            # A empty string means EOF and will not give rise to infinite loop
+            # since `''.isspace() == False`.
+            if line.isspace():
+                continue
+            break
+        return line  # empty string for EOF
+
+    def read_hash_line_2(self):
+        hash_line = self.read_hash_line()
+        if hash_line:
+            return parse_hash_line(hash_line)
+        return None  # None for EOF
+
+    def read_hash_line_3(self, *, root=None):
+        result = self.read_hash_line_2()
+        if result:
+            hex_hash_value, path_repr = result
+            path = _parse_path_repr(path_repr, root=root)
+            return hex_hash_value, path
+        return None  # None for EOF
+
+    def iter(self, mode=3, **kwargs):
+        if mode == 1:
+            read = self.read_hash_line
+        elif mode == 2:
+            read = self.read_hash_line_2
+        elif mode == 3:
+            read = self.read_hash_line_3
+        else:
+            ValueError("mode must in {{1, 2, 3}}, got {}".format(mode))
+
+        with self:
+            while True:
+                hash_line = read(**kwargs)
+                if not hash_line:
+                    break
+                yield hash_line
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        self.close()
+
+    def __iter__(self):
+        with self as f:
+            while True:
+                hash_line = f.read_hash_line()
+                if not hash_line:
+                    break
+                yield hash_line
+
+
+class HashFileWriter(object):
+    """General hash file writer.
+
+    Parameters
+    ----------
+    filepath : str or path-like
+        The path of a hash file.
+    """
+
+    def __init__(self, filepath):
+        self.name = filepath
+        self.file = open(filepath, "w", encoding="utf-8")
+
+    def close(self):
+        self.file.close()
+
+    def write_hash_line(self, hash_line):
+        """Write hash line.
+
+        Parameters
+        ----------
+        hash_line : str
+            A line of *hash* and *filename* with GNU Coreutils style.
+        """
+
+        self.file.write(hash_line)
+
+    def write_hash_line_2(self, hex_hash_value, path_repr):
+        hash_line = format_hash_line(hex_hash_value, path_repr)
+        self.write_hash_line(hash_line)
+
+    def write_hash_line_3(self, hex_hash_value, path, *, root=None):
+        path_repr = _format_path_repr(path, root=root)
+        self.write_hash_line_2(hex_hash_value, path_repr)
+
+    def write_comment(self, comment):
+        """Write comment.
+
+        Parameters
+        ----------
+        comment : str
+            A comment without leading # and tailing newline.
+        """
+
+        self.file.write("# ")
+        self.file.write(comment)
+        self.file.write("\n")
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        self.close()
+
+
+def format_hash_line(hex_hash_value, path_repr):
     r"""Format hash line.
 
     Parameters
     ----------
-    hash_value : bytes-like
-        The hash value.
-    path : str or path-like
-        The path of a file or a directory with corresponding hash value.
+    hex_hash_value : str
+        The hex hash value string.
+    path_repr : str or path-like
+        The certain path representation of a file or a directory with
+        corresponding hash value.
+        (1) A absolute path;
+        (2) A relative path;
+        (3) A path relative to a given root directory.
 
     Returns
     -------
     hash_line : str
-        The formatted `hash_value` and `path` with GNU Coreutils style.
+        A line of *hash* and *filename* with GNU Coreutils style.
 
     Examples
     --------
-    >>> hash_value = bytes.fromhex('d41d8cd98f00b204e9800998ecf8427e')
-    >>> path = 'a.txt'
-    >>> format_hash_line(hash_value, path)
+    >>> format_hash_line('d41d8cd98f00b204e9800998ecf8427e', 'a.txt')
     'd41d8cd98f00b204e9800998ecf8427e *a.txt\n'
     """
 
-    return "{} *{}\n".format(hash_value.hex(), path)
+    return "{} *{}\n".format(hex_hash_value, path_repr)
 
 
 def parse_hash_line(hash_line):
@@ -202,28 +343,41 @@ def parse_hash_line(hash_line):
     Parameters
     ----------
     hash_line : str
-        The formatted `hash_value` and `path` with GNU Coreutils style.
+        A line of *hash* and *filename* with GNU Coreutils style.
 
     Returns
     -------
-    hash_value : bytes
-        The hash value.
-    path : str
-        The path of a file or a directory with corresponding hash value.
+    hex_hash_value : str
+        The hex hash value string.
+    path_repr : str
+        The certain path representation of a file or a directory with
+        corresponding hash value.
+        (1) A absolute path;
+        (2) A relative path;
+        (3) A path relative to a given root directory.
 
     Examples
     --------
-    >>> hash_line = 'd41d8cd98f00b204e9800998ecf8427e *a.txt\n'
-    >>> hash_value, path = parse_hash_line(hash_line)
-    >>> hash_value.hex(), path
+    >>> parse_hash_line('d41d8cd98f00b204e9800998ecf8427e *a.txt\n')
     ('d41d8cd98f00b204e9800998ecf8427e', 'a.txt')
     """
 
     m = _HASH_LINE_RE.match(hash_line)
     if m is None:
         raise ParseHashLineError(hash_line)
-    hash_value, path = m.groups()
-    return bytes.fromhex(hash_value), path
+    return m.groups()
+
+
+def _format_path_repr(path, *, root=None):
+    if root is not None:
+        path = os.path.relpath(path, root)
+    return os.path.normpath(path)
+
+
+def _parse_path_repr(path_repr, *, root=None):
+    if root is not None:
+        path_repr = os.path.join(root, path_repr)
+    return os.path.normpath(path_repr)
 
 
 def generate_hash_line(path, hash_function, *, root=None):
@@ -242,13 +396,13 @@ def generate_hash_line(path, hash_function, *, root=None):
     Returns
     -------
     hash_line : str
-        The formatted `hash_value` and `path` with GNU Coreutils style.
+        A line of *hash* and *filename* with GNU Coreutils style.
     """
 
     hash_value = hash_function(path)
-    if root is not None:
-        path = os.path.normpath(os.path.relpath(path, root))
-    return format_hash_line(hash_value, path)
+    hex_hash_value = hash_value.hex()
+    path_repr = _format_path_repr(path, root=root)
+    return format_hash_line(hex_hash_value, path_repr)
 
 
 def check_hash_line(hash_line, hash_function, *, root=None):
@@ -257,7 +411,7 @@ def check_hash_line(hash_line, hash_function, *, root=None):
     Parameters
     ----------
     hash_line : str
-        The formatted `hash_value` and `path` with GNU Coreutils style.
+        A line of *hash* and *filename* with GNU Coreutils style.
     hash_function : callable(str or path-like) -> bytes-like
         A function for generating hash value.
     root : str, path-like or None, optional (default: None)
@@ -270,10 +424,51 @@ def check_hash_line(hash_line, hash_function, *, root=None):
         The path of a file or a directory with corresponding hash value.
     """
 
-    hash_value, path = parse_hash_line(hash_line)
-    if root is not None:
-        path = os.path.normpath(os.path.join(root, path))
+    hex_hash_value, path_repr = parse_hash_line(hash_line)
+    hash_value = bytes.fromhex(hex_hash_value)
+    path = _parse_path_repr(path_repr, root=root)
     curr_hash_value = hash_function(path)
     if not compare_digest(hash_value, curr_hash_value):
         raise CheckHashLineError(hash_line, hash_value, path, curr_hash_value)
     return path
+
+
+def _parse_for_re(hash_line, *, root=None):
+    hex_hash_value, path_repr = parse_hash_line(hash_line)
+    nameing_path = _parse_path_repr(path_repr, root=root)
+    hashing_path = os.path.join(os.path.dirname(nameing_path), hex_hash_value)
+    return hashing_path, nameing_path
+
+
+def re_name_to_hash(hash_line, *, root=None):
+    """Re name to hash.
+
+    Parameters
+    ----------
+    hash_line : str
+        A line of *hash* and *filename* with GNU Coreutils style.
+    root : str, path-like or None, optional (default: None)
+        The root directory of `path`. The path field in `hash_line` is relative
+        to the root directory.
+    """
+
+    hashing_path, nameing_path = _parse_for_re(hash_line, root=root)
+    os.rename(nameing_path, hashing_path)
+    return nameing_path, hashing_path
+
+
+def re_hash_to_name(hash_line, *, root=None):
+    """Re hash to name.
+
+    Parameters
+    ----------
+    hash_line : str
+        A line of *hash* and *filename* with GNU Coreutils style.
+    root : str, path-like or None, optional (default: None)
+        The root directory of `path`. The path field in `hash_line` is relative
+        to the root directory.
+    """
+
+    hashing_path, nameing_path = _parse_for_re(hash_line, root=root)
+    os.rename(hashing_path, nameing_path)
+    return hashing_path, nameing_path
