@@ -6,7 +6,14 @@ import click
 from click_option_group import MutuallyExclusiveOptionGroup
 
 from . import __version__
-from .core import CheckHashLineError, Hasher, check_hash_line, generate_hash_line
+from .core import (
+    CheckHashLineError,
+    Hasher,
+    HashFileReader,
+    HashFileWriter,
+    check_hash_line,
+    generate_hash_line,
+)
 from .utils.colorama import cprint, wrap_stream
 from .utils.glob import glob_scanner
 
@@ -45,18 +52,19 @@ class FilePath(click.Path):
 class Output(object):
     """Determine the output mode and provide the output interface."""
 
-    def __init__(self, sep, agg, null):
+    def __init__(self, sep=None, agg=None, null=None):
         if (sep and agg) or (sep and null) or (agg and null):
-            raise ValueError
+            raise ValueError("require exactly one argument")
 
         # Use the null mode by default.
         if not (sep or agg or null):
             null = True
 
+        # Determine the output mode and dump method.
         if sep:
             self._dump = self.output_sep
         elif agg:
-            self.agg_file = open(agg, "w", encoding="utf-8")
+            self.agg_file = HashFileWriter(agg)
             self._dump = self.output_agg
         elif null:
             self._dump = self.output_null
@@ -73,11 +81,11 @@ class Output(object):
         self._dump(hash_line, hash_path)
 
     def output_sep(self, hash_line, hash_path):
-        with open(hash_path, "w", encoding="utf-8") as f:
-            f.write(hash_line)
+        with HashFileWriter(hash_path) as f:
+            f.write_hash_line(hash_line)
 
     def output_agg(self, hash_line, hash_path):
-        self.agg_file.write(hash_line)
+        self.agg_file.write_hash_line(hash_line)
 
     @staticmethod
     def output_null(hash_line, hash_path):
@@ -149,26 +157,22 @@ class GetHash(object):
                 # The hash line already has a newline.
                 self.echo(hash_line, end="")
 
-    def _check_hash(self, hash_line, hash_path):
-        try:
-            root = self.check_root(hash_path)
-            path = check_hash_line(hash_line, self.hash_function, root=root)
-        except CheckHashLineError as e:
-            self.echo("[FAILURE] {}".format(e.path), fg="red")
-        except Exception as e:
-            self.echo_error(hash_path, e)
-        else:
-            self.echo("[SUCCESS] {}".format(path), fg="green")
+    def _check_hash(self, hash_path):
+        for hash_line in HashFileReader(hash_path):
+            try:
+                root = self.check_root(hash_path)
+                path = check_hash_line(hash_line, self.hash_function, root=root)
+            except CheckHashLineError as e:
+                self.echo("[FAILURE] {}".format(e.path), fg="red")
+            except Exception as e:
+                self.echo_error(hash_path, e)
+            else:
+                self.echo("[SUCCESS] {}".format(path), fg="green")
 
     def check_hash(self, patterns):
         for hash_path in glob_scanner(patterns, mode=self.glob_mode):
             try:
-                with open(hash_path, "r", encoding="utf-8") as f:
-                    # This function can process multiple hash lines.
-                    for hash_line in f:
-                        if hash_line.isspace():
-                            continue
-                        self._check_hash(hash_line, hash_path)
+                self._check_hash(hash_path)
             except Exception as e:
                 self.echo_error(hash_path, e)
 
@@ -188,9 +192,10 @@ class GetHash(object):
         self.close()
 
 
-def script_main(ctx, check, files, **options):
+def script_main(ctx, files, **options):
     """Generate the body for the main function."""
 
+    check = options.pop("check", False)
     # Convert bool flags to streams.
     no_stdout = options.pop("no_stdout", False)
     no_stderr = options.pop("no_stderr", False)
@@ -274,7 +279,12 @@ def gethashcli(name, suffix):
             is_flag=True,
             help="Do not output to files. This is the default output mode.",
         )
-        @click.option("--tqdm-ascii", type=click.BOOL, default=True, show_default=True)
+        @click.option(
+            "--tqdm-ascii",
+            type=click.BOOL,
+            default=(sys.platform == "win32"),
+            show_default=True,
+        )
         @click.option(
             "--tqdm-disable", type=click.BOOL, default=False, show_default=True
         )
