@@ -2,7 +2,7 @@ __all__ = ["strxor"]
 
 
 def _strxor(b1, b2):
-    return (x1 ^ x2 for x1, x2 in zip(b1, b2))
+    return bytes(x1 ^ x2 for x1, x2 in zip(b1, b2))
 
 
 try:
@@ -24,7 +24,7 @@ def strxor(b1, b2):
     Returns
     -------
     result : bytes
-        The result of b1 xor b2.
+        The result of `b1` xor `b2`.
     """
 
     return _strxor(b1, b2)
